package com.hzya.frame.plugin.lets.dingtalk.utils;

import cn.hutool.core.lang.Assert;
import com.aliyun.dingtalkstorage_1_0.models.AddPermissionResponse;
import com.aliyun.dingtalkstorage_1_0.models.CommitFileRequest;
import com.aliyun.dingtalkstorage_1_0.models.CommitFileResponse;
import com.aliyun.dingtalkstorage_1_0.models.GetFileUploadInfoResponse;
import com.hzya.frame.plugin.lets.dingtalk.vo.FileModuleVO;
import com.hzya.frame.plugin.lets.dingtalk.vo.SysFileVO;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.List;
import java.util.Map;

public class FileUtil {
    protected static Logger logger = LogManager.getLogger(FileUtil.class);

    @Value("${DING.SPACEID}")
    private static String spaceId;

    /**
     * 审批文件上传钉盘
     * @param accessToken 应用Token
     * @param unionid 钉钉用户unionid
     * @param dingFileUrlID 钉盘审批实例中附件地址
     * @param fileVO 的文件信息
     * @param stream 文件读取的字节流
     * @return
     */
    public static FileModuleVO get4DingDileModuleVO(String accessToken, String unionid, String dingFileUrlID, SysFileVO fileVO, ByteArrayOutputStream stream){
        FileModuleVO fileModuleVO = new FileModuleVO();
        try {
            //钉钉--上传附件
            //1.调用服务端API-获取审批钉盘空间信息接口，获取审批空间spaceId。
            System.out.println(spaceId);
            //2.调用服务端API-获取文件上传信息接口，获取审批空间文件上传信息并执行上传。
            /////////////////////////////////////////////////////////////////////////////////////////////////此处很容易调权限，钉钉问题。添加权限
            AddPermissionResponse addPermissionResponse = DingTalkUtils.addFileStorageByUnionId(accessToken, spaceId, unionid);
            if(addPermissionResponse.getBody().getSuccess()!=true){
                Assert.state(false,"钉钉添加审批实例附件到钉盘权限失败");
            }
            /////////////////////////////////////////////////////////////////////////////////////////////////此处很容易调权限，钉钉问题。添加权限
            //2.1调用本接口，获取上传文件需要的resourceUrls和headers参数值。
            GetFileUploadInfoResponse headerSignature = DingTalkUtils.getFileUploadInfo(accessToken, spaceId, unionid, "HEADER_SIGNATURE", false);
            String uploadKey = headerSignature.getBody().getUploadKey();
            List<String> resourceUrls = headerSignature.getBody().getHeaderSignatureInfo().getResourceUrls();
            if (resourceUrls.size() > 1) {
                Assert.state(false, "获取文件上传信息接口中resourceUrls不唯一！！！");
            }
            Map<String, String> headers = headerSignature.getBody().getHeaderSignatureInfo().getHeaders();
            //2.2使用OSS的header加签方式上传文件，参考示例：
            // 从接口返回信息中拿到headers
            URL url = new URL(resourceUrls.get(0));
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            if (headers != null) {
                for (Map.Entry<String, String> entry : headers.entrySet()) {
                    connection.setRequestProperty(entry.getKey(), entry.getValue());
                }
            }
            connection.setDoOutput(true);
            connection.setRequestMethod("PUT");
            connection.setUseCaches(false);
            connection.setReadTimeout(10000);
            connection.setConnectTimeout(10000);
            connection.connect();
            OutputStream out = connection.getOutputStream();
            out.write(stream.toByteArray());
            out.flush();
            out.close();
            int responseCode = connection.getResponseCode();
            connection.disconnect();
            if (responseCode == 200) {
                logger.info("审批实例附件上传审批钉盘成功,UploadKey:-->" + headerSignature.getBody().getUploadKey());
            } else {
                Assert.state(false,"审批实例附件上传审批钉盘失败");
            }
            //2.3调用提交文件接口，完成文件上传。
            CommitFileRequest.CommitFileRequestOptionAppProperties commitFileRequestOptionAppProperties = new CommitFileRequest.CommitFileRequestOptionAppProperties();
            commitFileRequestOptionAppProperties.setName(fileVO.getFileName());
            commitFileRequestOptionAppProperties.setValue(fileVO.getFileName());
            commitFileRequestOptionAppProperties.setVisibility("PUBLIC");//属性可见性。 - PUBLIC：所有应用都可见 - PRIVATE：仅限当前应用可见

            //3.提交文件
            CommitFileResponse commitFileResponse = DingTalkUtils.submitFile(accessToken, spaceId, unionid, uploadKey, fileVO.getFileName(), dingFileUrlID, commitFileRequestOptionAppProperties);

            //钉盘文件唯一id
            String dingFileUniqueId = commitFileResponse.getBody().getDentry().getId();

            //4.组装钉钉文件控件-->"[{\"spaceId\": \"163xxxx658\", \"fileName\": \"2644.JPG\", \"fileSize\": \"333\", \"fileType\": \"jpg\", \"fileId\": \"643xxxx140\"}]"
            fileModuleVO.setSpaceId(spaceId);
            fileModuleVO.setFileName(fileVO.getFileName());
            fileModuleVO.setFileSize(String.valueOf(fileVO.getFileSize()));
            String s = StringUtils.substringAfter(fileVO.getFileName(), ".");
            fileModuleVO.setFileType(s);//取后缀
            fileModuleVO.setFileId(dingFileUniqueId);

        }catch (Exception e){
            e.printStackTrace();
            logger.error(e);
            Assert.state(false,"文件存钉盘失败,失败信息:{}",e.getMessage());
        }
        return fileModuleVO;
    }
}
